@@ -80,6 +80,12 @@ for(i in (1:nK)){
  if(pHscale[i]=="F"){factor[i] <- kconv(S=S[i], T=T[i], P=P[i])$kSWS2free ; pHsc[i] <- "free scale"}
  if(pHscale[i]=="SWS"){factor[i] <- 1 ; pHsc[i] <- "seawater scale"}
 Ksi[i] <- Ksi[i]*factor[i]
+}
+
+##------------Warnings
+
+for(i in 1:nK){
+if((T[i]>45)|(S[i]>45)|(T[i]<0)|(S[i]<0)){warning("S and/or T is outside the range of validity of the formulation available for Ksi in seacarb.")}
 }
 
   attr(Ksi,"unit")     = "mol/kg-soln"
