@@ -13,9 +13,9 @@
 #
 #
 carb<-
-function(flag, var1, var2, S=35, T=25, P=0, Pt=0, Sit=0, k1k2='l', kf='pf', pHscale="T"){
+function(flag, var1, var2, S=35, T=25, P=0, Pt=0, Sit=0, k1k2='x', kf='x', ks="d", pHscale="T"){
 RES <- data.frame()
-n <- max(length(var1), length(var2), length(S), length(T), length(P), length(Pt), length(Sit), length(k1k2), length(kf), length(pHscale))
+n <- max(length(var1), length(var2), length(S), length(T), length(P), length(Pt), length(Sit), length(k1k2), length(kf), length(pHscale), length(ks))
 if(length(flag)!=n){ flag <- rep(flag[1],n)}
 if(length(var1)!=n){ var1 <- rep(var1[1],n)}
 if(length(var2)!=n){ var2 <- rep(var2[1],n)}
@@ -26,9 +26,11 @@ if(length(Pt)!=n){ Pt <- rep(Pt[1],n)}
 if(length(Sit)!=n){ Sit <- rep(Sit[1],n)}
 if(length(k1k2)!=n){ k1k2 <- rep(k1k2[1],n)}
 if(length(kf)!=n){ kf <- rep(kf[1],n)}
+if(length(ks)!=n){ ks <- rep(ks[1],n)}
 if(length(pHscale)!=n){pHscale <- rep(pHscale[1],n)}
 
-df <- data.frame(flag, var1, var2, S, T, P, Pt, Sit)
+df <- data.frame(flag, var1, var2, S, T, P, Pt, Sit, pHscale)
+
 
 ##BOUCLE
 for(i in (1:nrow(df))) {
@@ -40,7 +42,11 @@ for(i in (1:nrow(df))) {
   P <- as.numeric(df[i,6])
   Pt <- as.numeric(df[i,7])
   Sit <- as.numeric(df[i,8])
+  pHscale <- as.character(df[i,9])
 
+res <- rep(NA, 14)
+
+if((is.na(var1)==FALSE)&(is.na(var2)==FALSE)){
 	
 #-------Constantes----------------
 
@@ -63,17 +69,18 @@ fluo = (7*(S/35))*1e-5        # (mol/kg), DOE94 fluoride total
 #--------------------- calcul des K ----------------------------------
 #---------------------------------------------------------------------
 
-K1 <- K1(S=S, T=T, P=P, pHscale=pHscale[i], k1k2=k1k2[i])   
-K2 <- K2(S=S, T=T, P=P, pHscale=pHscale[i], k1k2=k1k2[i])
-Kf <- Kf(S=S, T=T, P=P, pHscale=pHscale[i], kf=kf[i])
-Ks <- Ks(S=S, T=T, P=P)
-Kw <- Kw(S=S, T=T, P=P, pHscale=pHscale[i])
+
+K1 <- K1(S=S, T=T, P=P, pHscale=pHscale, k1k2=k1k2[i])   
+K2 <- K2(S=S, T=T, P=P, pHscale=pHscale, k1k2=k1k2[i])
+Kf <- Kf(S=S, T=T, P=P, pHscale=pHscale, kf=kf[i])
+Ks <- Ks(S=S, T=T, P=P, ks=ks[i])
+Kw <- Kw(S=S, T=T, P=P, pHscale=pHscale)
 Kh <- Kh(S=S, T=T, P=P)
-Kb <- Kb(S=S, T=T, P=P, pHscale=pHscale[i])
-K1p <- K1p(S=S, T=T, P=P, pHscale=pHscale[i])
-K2p <- K2p(S=S, T=T, P=P, pHscale=pHscale[i])
-K3p <- K3p(S=S, T=T, P=P, pHscale=pHscale[i])
-Ksi <- Ksi(S=S, T=T, P=P, pHscale=pHscale[i])
+Kb <- Kb(S=S, T=T, P=P, pHscale=pHscale)
+K1p <- K1p(S=S, T=T, P=P, pHscale=pHscale)
+K2p <- K2p(S=S, T=T, P=P, pHscale=pHscale)
+K3p <- K3p(S=S, T=T, P=P, pHscale=pHscale)
+Ksi <- Ksi(S=S, T=T, P=P, pHscale=pHscale)
 Kspa <- Kspa(S=S, T=T, P=P)
 Kspc <- Kspc(S=S, T=T, P=P)
 	
@@ -160,13 +167,23 @@ rho <- rho(S=S,T=T,P=P)
 	hpo4 <- Pt*K1p*K2p*x/(x^3+K1p*x^2+K1p*K2p*x+K1p*K2p*K3p)
 	po4 <- Pt*K1p*K2p*K3p/(x^3+K1p*x^2+K1p*K2p*x+K1p*K2p*K3p)
 	siooh3 <- Sit/(1+x/Ksi)
-	h <- x/(1+ST/Ks)
-	hso4 <- ST/(1+Ks/h)
-	hf <- fluo/(1+Kf/x)
-	############
-	OUT <- hco3+2*co3+boh4+oh+hpo4+2*po4+siooh3-h-hso4-hf-h3po4-ALK
+	
+	## calculate Hfree and Htot
+  if(pHscale=="F"){hfree <- x  ## if pHscale = free scale
+	htot <- 10^(-pHconv(flag=2, pH=(-log10(x)), S=S, T=T, P=P))}   
+	if(pHscale=="T"){hfree <- 10^(-pHconv(flag=4, pH=(-log10(x)), S=S, T=T, P=P))
+  htot <- x}
+	if(pHscale=="SWS"){hfree <- 10^(-pHconv(flag=5, pH=(-log10(x)), S=S, T=T, P=P))
+	htot <- 10^(-pHconv(flag=1, pH=(-log10(x)), S=S, T=T, P=P))}
+	
+	hso4 <- ST/(1+Ks/hfree)
+	hf <- fluo/(1+Kf/htot)
+	
+  ############
+	OUT <- hco3+2*co3+boh4+oh+hpo4+2*po4+siooh3-hfree-hso4-hf-h3po4-ALK
 	OUT}	
-	h <- uniroot(fALK,c(10^(-9.5),10^(-3.5)), tol=1e-20)$root
+	h <- uniroot(fALK,c(10^(-9.5),10^(-3.5)), tol=1e-20)$root	
+	
 	DIC <- CO2*(1+K1/h+K1*K2/(h*h))
 	HCO3 <- (DIC*K1*h)/(h*h+K1*h+K1*K2)
 	CO3 <- (DIC*K1*K2)/(h*h+K1*h+K1*K2)
@@ -231,11 +248,18 @@ rho <- rho(S=S,T=T,P=P)
 	hpo4 <- Pt*K1p*K2p*h/(h^3+K1p*h^2+K1p*K2p*h+K1p*K2p*K3p)
 	po4 <- Pt*K1p*K2p*K3p/(h^3+K1p*h^2+K1p*K2p*h+K1p*K2p*K3p)
 	siooh3 <- Sit/(1+h/Ksi)
-	hff <- h/(1+ST/Ks)
-	hso4 <- ST/(1+Ks/hff)
-	hf <- fluo/(1+Kf/h)
+	## calculate Hfree anf Htot
+  if(pHscale=="F"){hfree <- h  ## if pHscale = free scale
+	htot <- 10^(-pHconv(flag=2, pH=(-log10(h)), S=S, T=T, P=P))}   
+	if(pHscale=="T"){hfree <- 10^(-pHconv(flag=4, pH=(-log10(h)), S=S, T=T, P=P))
+  htot <- h}
+	if(pHscale=="SWS"){hfree <- 10^(-pHconv(flag=5, pH=(-log10(h)), S=S, T=T, P=P))
+	htot <- 10^(-pHconv(flag=1, pH=(-log10(h)), S=S, T=T, P=P))}
+	
+	hso4 <- ST/(1+Ks/hfree)
+	hf <- fluo/(1+Kf/htot)
 	############
-	OUT <- hco3+2*co3+boh4+oh+hpo4+2*po4+siooh3-h-hso4-hf-h3po4-ALK
+	OUT <- hco3+2*co3+boh4+oh+hpo4+2*po4+siooh3-hfree-hso4-hf-h3po4-ALK
 	OUT}	
 	DIC <- uniroot(fALK,c(5e-4,0.8), tol=1e-20)$root
 	CO2 <- DIC/(1+K1/h+K1*K2/(h^2))
@@ -285,14 +309,23 @@ rho <- rho(S=S,T=T,P=P)
 	hpo4 <- Pt*K1p*K2p*x/(x^3+K1p*x^2+K1p*K2p*x+K1p*K2p*K3p)
 	po4 <- Pt*K1p*K2p*K3p/(x^3+K1p*x^2+K1p*K2p*x+K1p*K2p*K3p)
 	siooh3 <- Sit/(1+x/Ksi)
-	h <- x/(1+ST/Ks)
-	hso4 <- ST/(1+Ks/h)
-	hf <- fluo/(1+Kf/x)
+	
+	## calculate Hfree anf Htot
+  if(pHscale=="F"){hfree <- x  ## if pHscale = free scale
+	htot <- 10^(-pHconv(flag=2, pH=(-log10(x)), S=S, T=T, P=P))}   
+	if(pHscale=="T"){hfree <- 10^(-pHconv(flag=4, pH=(-log10(x)), S=S, T=T, P=P))
+  htot <- x}
+	if(pHscale=="SWS"){hfree <- 10^(-pHconv(flag=5, pH=(-log10(x)), S=S, T=T, P=P))
+	htot <- 10^(-pHconv(flag=1, pH=(-log10(x)), S=S, T=T, P=P))}
+	
+	hso4 <- ST/(1+Ks/hfree)
+	hf <- fluo/(1+Kf/htot)
 	############
-	OUT <- hco3+2*co3+boh4+oh+hpo4+2*po4+siooh3-h-hso4-hf-h3po4-ALK
+	OUT <- hco3+2*co3+boh4+oh+hpo4+2*po4+siooh3-hfree-hso4-hf-h3po4-ALK
 	OUT}
 
 	h <- uniroot(fALK,c(10^(-9.5),10^(-3)),tol=1e-20)$root
+ 
 	CO2 <- h*HCO3/K1
 	CO3 <- K2*HCO3/h
 	DIC <- CO2 + HCO3 + CO3
@@ -333,14 +366,24 @@ rho <- rho(S=S,T=T,P=P)
 	hpo4 <- Pt*K1p*K2p*x/(x^3+K1p*x^2+K1p*K2p*x+K1p*K2p*K3p)
 	po4 <- Pt*K1p*K2p*K3p/(x^3+K1p*x^2+K1p*K2p*x+K1p*K2p*K3p)
 	siooh3 <- Sit/(1+x/Ksi)
-	h <- x/(1+ST/Ks)
-	hso4 <- ST/(1+Ks/h)
-	hf <- fluo/(1+Kf/x)
+	
+	## calculate Hfree anf Htot
+  if(pHscale=="F"){hfree <- x  ## if pHscale = free scale
+	htot <- 10^(-pHconv(flag=2, pH=(-log10(x)), S=S, T=T, P=P))}   
+	if(pHscale=="T"){hfree <- 10^(-pHconv(flag=4, pH=(-log10(x)), S=S, T=T, P=P))
+  htot <- x}
+	if(pHscale=="SWS"){hfree <- 10^(-pHconv(flag=5, pH=(-log10(x)), S=S, T=T, P=P))
+	htot <- 10^(-pHconv(flag=1, pH=(-log10(x)), S=S, T=T, P=P))}
+	
+	hso4 <- ST/(1+Ks/hfree)
+	hf <- fluo/(1+Kf/htot)
+	
 	############
-	OUT <- hco3+2*co3+boh4+oh+hpo4+2*po4+siooh3-h-hso4-hf-h3po4-ALK
+	OUT <- hco3+2*co3+boh4+oh+hpo4+2*po4+siooh3-hfree-hso4-hf-h3po4-ALK
 	OUT}
 	h <- uniroot(fALK,c(10^(-9.5),10^(-3.5)),tol=1e-20)$root
-	HCO3 <- h*CO3/K2
+	
+ 	HCO3 <- h*CO3/K2
 	CO2 <- h*HCO3/K1
 	fCO2 <- CO2/Kh
 	DIC <- HCO3+CO2+CO3
@@ -375,15 +418,24 @@ rho <- rho(S=S,T=T,P=P)
 	hpo4 <- Pt*K1p*K2p*x/(x^3+K1p*x^2+K1p*K2p*x+K1p*K2p*K3p)
 	po4 <- Pt*K1p*K2p*K3p/(x^3+K1p*x^2+K1p*K2p*x+K1p*K2p*K3p)
 	siooh3 <- Sit/(1+x/Ksi)
-	h <- x/(1+ST/Ks)
-	hso4 <- ST/(1+Ks/h)
-	hf <- fluo/(1+Kf/x)
+	
+	## calculate Hfree and Htot
+  if(pHscale=="F"){hfree <- x  ## if pHscale = free scale
+	htot <- 10^(-pHconv(flag=2, pH=(-log10(x)), S=S, T=T, P=P))}   
+	if(pHscale=="T"){hfree <- 10^(-pHconv(flag=4, pH=(-log10(x)), S=S, T=T, P=P))
+  htot <- x}
+	if(pHscale=="SWS"){hfree <- 10^(-pHconv(flag=5, pH=(-log10(x)), S=S, T=T, P=P))
+	htot <- 10^(-pHconv(flag=1, pH=(-log10(x)), S=S, T=T, P=P))}
+	
+	hso4 <- ST/(1+Ks/hfree)
+	hf <- fluo/(1+Kf/htot)
+	
 	############
-	OUT <- hco3+2*co3+boh4+oh+hpo4+2*po4+siooh3-h-hso4-hf-h3po4-ALK
+	OUT <- hco3+2*co3+boh4+oh+hpo4+2*po4+siooh3-hfree-hso4-hf-h3po4-ALK
 	OUT}
 
-	h <- uniroot(fALK,c(1e-10,10^(-3.5)),tol=1e-30)$root
-	
+	h <- uniroot(fALK,c(1e-10,10^(-3.5)),tol=1e-30)$root	
+ 
 	HCO3 <- (DIC*K1*h)/(h*h+K1*h+K1*K2)
 	CO3 <- (DIC*K1*K2)/(h*h+K1*h+K1*K2)
 	CO2 <- h*HCO3/K1
@@ -460,13 +512,24 @@ rho <- rho(S=S,T=T,P=P)
 	hpo4 <- Pt*K1p*K2p*x/(x^3+K1p*x^2+K1p*K2p*x+K1p*K2p*K3p)
 	po4 <- Pt*K1p*K2p*K3p/(x^3+K1p*x^2+K1p*K2p*x+K1p*K2p*K3p)
 	siooh3 <- Sit/(1+x/Ksi)
-	h <- x/(1+ST/Ks)
-	hso4 <- ST/(1+Ks/h)
-	hf <- fluo/(1+Kf/x)
+	
+	## calculate Hfree and Htot
+  if(pHscale=="F"){hfree <- x  ## if pHscale = free scale
+	htot <- 10^(-pHconv(flag=2, pH=(-log10(x)), S=S, T=T, P=P))}   
+	if(pHscale=="T"){hfree <- 10^(-pHconv(flag=4, pH=(-log10(x)), S=S, T=T, P=P))
+  htot <- x}
+	if(pHscale=="SWS"){hfree <- 10^(-pHconv(flag=5, pH=(-log10(x)), S=S, T=T, P=P))
+	htot <- 10^(-pHconv(flag=1, pH=(-log10(x)), S=S, T=T, P=P))}
+	
+	
+	hso4 <- ST/(1+Ks/hfree)
+	hf <- fluo/(1+Kf/htot)
 	############
-	OUT <- hco3+2*co3+boh4+oh+hpo4+2*po4+siooh3-h-hso4-hf-h3po4-ALK
+	OUT <- hco3+2*co3+boh4+oh+hpo4+2*po4+siooh3-hfree-hso4-hf-h3po4-ALK
 	OUT}	
+	
 	h <- uniroot(fALK,c(1e-10,10^(-3.5)),tol=1e-20)$root
+   
 	HCO3 <- K1*CO2/h
 	CO3 <- K2*HCO3/h
 	PH <- -log10(h)
@@ -486,9 +549,9 @@ rho <- rho(S=S,T=T,P=P)
 	}
 
 	# ------------ CALCULATION OF ALK in cases 
-	Ks <- Ks(S=S, T=T, P=P)
+  Ks <- Ks(S=S, T=T, P=P)
 	cases <- c(1, 2, 3, 5, 6, 7, 9, 10, 12, 14, 21, 22, 23, 24, 25)
-	if (length(which(cases==flag))==1){
+	if (flag %in% cases){
 	x <- h
 	hco3 <- DIC*x*K1/(x*x + K1*x + K1*K2)
 	co3 <- DIC*K1*K2/(x*x + K1*x + K1*K2)
@@ -498,11 +561,18 @@ rho <- rho(S=S,T=T,P=P)
 	hpo4 <- Pt*K1p*K2p*x/(x^3+K1p*x^2+K1p*K2p*x+K1p*K2p*K3p)
 	po4 <- Pt*K1p*K2p*K3p/(x^3+K1p*x^2+K1p*K2p*x+K1p*K2p*K3p)
 	siooh3 <- Sit/(1+x/Ksi)
-	h <- x/(1+ST/Ks)
-	hso4 <- ST/(1+Ks/h)
-	hf <- fluo/(1+Kf/x)
-	ALK <- hco3+2*co3+boh4+oh+hpo4+2*po4+siooh3-h-hso4-hf-h3po4
+	if(pHscale=="F"){hfree <- x  ## if pHscale = free scale
+	htot <- 10^(-pHconv(flag=2, pH=(-log10(x)), S=S, T=T, P=P))}
+	if(pHscale=="T"){hfree <- 10^(-pHconv(flag=4, pH=(-log10(x)), S=S, T=T, P=P))
+  htot <- x}
+	if(pHscale=="SWS"){hfree <- 10^(-pHconv(flag=5, pH=(-log10(x)), S=S, T=T, P=P))
+  htot <- 10^(-pHconv(flag=1, pH=(-log10(x)), S=S, T=T, P=P))}
+	# h is the concentration in H+ at the free scale
+	hso4 <- ST/(1+Ks/hfree)
+	hf <- fluo/(1+Kf/htot)
+	ALK <- hco3+2*co3+boh4+oh+hpo4+2*po4+siooh3-hfree-hso4-hf-h3po4
 	}
+    
 
 	##########################################################
 	# CALCULATION OF ARAGONITE AND CALCITE SATURATION STATE  #
@@ -515,7 +585,9 @@ rho <- rho(S=S,T=T,P=P)
 	pCO2 <- pCO2*1e6
 	fCO2 <- fCO2*1e6	
 
-	res<-data.frame(flag,S,T,P,PH,CO2,pCO2,fCO2,HCO3,CO3,DIC,ALK,Oa,Oc)
+	res <- data.frame(flag,S,T,P,PH,CO2,pCO2,fCO2,HCO3,CO3,DIC,ALK,Oa,Oc)
+	
+	}
 	RES<- rbind(RES, res)
 	}
 	names(RES) <- c("flag", "S", "T", "P", "pH", "CO2", "pCO2", "fCO2", "HCO3", "CO3", "DIC", "ALK", "OmegaAragonite", "OmegaCalcite")
